/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.service;

import com.sun.istack.NotNull;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.ulstu.extractor.model.Author;
import ru.ulstu.extractor.model.Branch;
import ru.ulstu.extractor.model.Commit;
import ru.ulstu.extractor.model.Repository;
import ru.ulstu.extractor.repository.AuthorRepository;
import ru.ulstu.extractor.repository.BranchRepository;
import ru.ulstu.extractor.repository.CommitRepository;
import ru.ulstu.extractor.repository.RepositoryRepository;

import java.io.IOException;
import java.util.List;

@Service
public class IndexService {
    private final GitRepositoryService gitRepositoryService;
    private final RepositoryRepository repositoryRepository;
    private final BranchRepository branchRepository;
    private final CommitRepository commitRepository;
    private final AuthorRepository authorRepository;


    public IndexService(GitRepositoryService gitRepositoryService,
                        RepositoryRepository repositoryRepository,
                        BranchRepository branchRepository,
                        CommitRepository commitRepository,
                        AuthorRepository authorRepository) {
        this.gitRepositoryService = gitRepositoryService;
        this.repositoryRepository = repositoryRepository;
        this.branchRepository = branchRepository;
        this.commitRepository = commitRepository;
        this.authorRepository = authorRepository;
    }

    @Transactional
    public void index(@NotNull String repositoryUrl, @NotNull String branchName) throws GitAPIException, IOException {
        Repository repository = repositoryRepository.findByUrl(repositoryUrl);
        if (repository == null) {
            repositoryRepository.save(new Repository(repositoryUrl));
        }
        Branch branch = branchRepository.findByRepositoryAndName(repository, branchName);
        if (branch == null) {
            branch = branchRepository.save(new Branch(repository, branchName));
        }
        List<Commit> commits = gitRepositoryService.getCommits(repositoryUrl, branchName);
        commitRepository.deleteAll(branch.getCommits());
        branch.getCommits().clear();
        branch.setCommits(commits);
        branchRepository.save(branch);
    }

    public List<Author> getRepositoryAuthors(@NotNull String repositoryUrl, @NotNull String branchName) {
        return authorRepository.findByRepositoryAndBranch(repositoryRepository.findByUrl(repositoryUrl), branchName);
    }

    public Page<Commit> getCommits(@NotNull String repositoryUrl, @NotNull String branchName, Pageable pageable) {
        return commitRepository.findByRepositoryAndBranch(pageable, repositoryRepository.findByUrl(repositoryUrl), branchName);
    }
}
